# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from zuora_sdk.models.create_payment_schedule_request_items import CreatePaymentScheduleRequestItems
from zuora_sdk.models.payment_schedule_billing_document import PaymentScheduleBillingDocument
from zuora_sdk.models.payment_schedule_payment_option_fields import PaymentSchedulePaymentOptionFields
from typing import Optional, Set
from typing_extensions import Self

class CreatePaymentScheduleRequest(BaseModel):
    """
    CreatePaymentScheduleRequest
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    account_id: Optional[StrictStr] = Field(default=None, description="ID of the customer account the payment schedule belongs to.  **Note:** `accountId` and `accountNumber` cannot both be `null`. When both fields are specified, the two values must match each other. ", alias="accountId")
    account_number: Optional[StrictStr] = Field(default=None, description="Account number of the customer account the payment schedule belongs to.  **Note:** `accountId` and `accountNumber` cannot both be `null`. When both fields are specified, the two values must match each other. ", alias="accountNumber")
    amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The amount of each payment schedule item in the payment schedule.  **Note:** - This field is required when `items` is not specified. - This field will be ignored when `items` is specified. - When creating recurring payment schedules, there are 2 options to specify amounts:     - Specify `totalAmount` and `occurrences`, `amount` will be calculated.   - Specify `amount` and `occurrences`, `totalAmount` will be calculated.   You must specify either `totalAmount` or `amount`. Specifying both fields at the same time is not allowed. ")
    billing_document: Optional[PaymentScheduleBillingDocument] = Field(default=None, alias="billingDocument")
    currency: Optional[StrictStr] = Field(default=None, description="Currency of the payment schedule.  **Note:** - This field is optional. The default value is the account's default currency. - This field will be ignored when `items` is specified. ")
    description: Optional[StrictStr] = Field(default=None, description="Description of the payment schedule. Max length is 255. ")
    items: Optional[List[CreatePaymentScheduleRequestItems]] = Field(default=None, description="Container array for payment schedule items. ")
    occurrences: Optional[StrictInt] = Field(default=None, description="The number of payment schedule item to be created. Maximum value is 1000.  **Note:** - This field is required when `items` is not specified. - This field will be ignored when `items` is specified. ")
    payment_gateway_id: Optional[StrictStr] = Field(default=None, description="ID of the payment gateway.  **Note:** - This field is optional. The default value is the account's default payment gateway ID. If no payment gateway ID is found on the cusotmer account level, the default value will be the tenant's default payment gateway ID. - This field will be ignored when `items` is specified. ", alias="paymentGatewayId")
    payment_method_id: Optional[StrictStr] = Field(default=None, description="ID of the payment method.  **Note:** - This field is optional. The default value is the account's default payment method ID. - This field will be ignored when `items` is specified. ", alias="paymentMethodId")
    payment_option: Optional[List[PaymentSchedulePaymentOptionFields]] = Field(default=None, description="Container for the paymentOption items, which describe the transactional level rules for processing payments. Currently, only the Gateway Options type is supported.   Here is an example: ``` \"paymentOption\": [   {     \"type\": \"GatewayOptions\",     \"detail\": {       \"SecCode\":\"WEB\"     }   } ] ```  `paymentOption` of the payment schedule takes precedence over `paymentOption` of the payment schedule item.  To enable this field, submit a request at [Zuora Global Support](https://support.zuora.com/). This field is only available if `zuora-version` is set to `337.0` or later. ", alias="paymentOption")
    payment_schedule_number: Optional[StrictStr] = Field(default=None, description="You can use this field to specify the number of the payment schedule. Only characters from the following sets are allowed: A-Z, a-z, 0-9, and `-`.  Payment numbers must start with a letter. In addition,`-` can only be used at most once and cannot be placed at the beginning or the end of the payment numbers. ", alias="paymentScheduleNumber")
    period: Optional[StrictStr] = Field(default=None, description="The frequency for the payment collection since the `startDate`.  **Note:** - Thie field is required when `items` is not specified. - This field will be ignored when `items` is specified. - If `startDate` is `30` or `31` and `period` is `Monthly`, when in February, payment schedule will use the last day of February for payment collection. ")
    prepayment: Optional[StrictBool] = Field(default=None, description="Indicates whether the payments created by the payment schedule will be used as reserved payments. This field will only be available if the prepaid cash drawdown permission is enabled. See [Prepaid Cash with Drawdown](https://knowledgecenter.zuora.com/Zuora_Billing/Billing_and_Invoicing/JA_Advanced_Consumption_Billing/Prepaid_Cash_with_Drawdown) for more information. ")
    run_hour: Optional[StrictInt] = Field(default=None, description="Specifies at which hour in the day in the tenant’s time zone when this payment will be collected. Available values: `[0,1,2,~,22,23]`.  **Note:** - If the time difference between your tenant’s timezone and the timezone where Zuora servers are is not in full hours, for example, 2.5 hours, the payment schedule items will be triggered half hour later than your scheduled time. - If the payment `runHour` and `scheduledDate` are backdated, the system will collect the payment when the next runHour occurs. - This field is optional. The default value is `0`. - This field will be ignored when `items` is specified. ", alias="runHour")
    standalone: Optional[StrictBool] = Field(default=None, description="Indicate whether the payments created by the payment schedule are standalone payments or not. When setting to `true`, standalone payments will be created. When setting to `false`, you can either specify a billing document, or not specifying any billing documents. In the later case, unapplied payments will be created. If set to `null`, standalone payment will be created.  **Note**:  - This field is only available if the Standalone Payment is enabled. Do not include this field if Standalone Payment is not enabled. - If Standalone Payment is enabled, default value is `true`. ")
    start_date: Optional[date] = Field(default=None, description="The date for the first payment collection.  **Note:** - This field is required when `items` is not specified. - This field will be ignored when `items` is specified. ", alias="startDate")
    total_amount: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The total amount of that the payment schedule will collect. This field is only available for recurring payment schedules.   **Note**: - When creating recurring payment schedules, there are 2 options to specify amounts:      - Specify `totalAmount` and `occurrences`, `amount` will be calculated.   - Specify `amount` and `occurrences`, `totalAmount` will be calculated.      You must specify either `totalAmount` or `amount`. Specifying both fields at the same time is not allowed. - If the Standalone Payments feature is enabled and `standalone` is set to `true` for the payment schedule, `totalAmount` will be ignored. ", alias="totalAmount")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["accountId", "accountNumber", "amount", "billingDocument", "currency", "description", "items", "occurrences", "paymentGatewayId", "paymentMethodId", "paymentOption", "paymentScheduleNumber", "period", "prepayment", "runHour", "standalone", "startDate", "totalAmount"]

    @field_validator('period')
    def period_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['Monthly', 'Weekly', 'BiWeekly']):
            raise ValueError("must be one of enum values ('Monthly', 'Weekly', 'BiWeekly')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreatePaymentScheduleRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of billing_document
        if self.billing_document:
            _dict['billingDocument'] = self.billing_document.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in items (list)
        _items = []
        if self.items:
            for _item_items in self.items:
                if _item_items:
                    _items.append(_item_items.to_dict())
            _dict['items'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in payment_option (list)
        _items = []
        if self.payment_option:
            for _item_payment_option in self.payment_option:
                if _item_payment_option:
                    _items.append(_item_payment_option.to_dict())
            _dict['paymentOption'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreatePaymentScheduleRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "accountId": obj.get("accountId"),
            "accountNumber": obj.get("accountNumber"),
            "amount": obj.get("amount"),
            "billingDocument": PaymentScheduleBillingDocument.from_dict(obj["billingDocument"]) if obj.get("billingDocument") is not None else None,
            "currency": obj.get("currency"),
            "description": obj.get("description"),
            "items": [CreatePaymentScheduleRequestItems.from_dict(_item) for _item in obj["items"]] if obj.get("items") is not None else None,
            "occurrences": obj.get("occurrences"),
            "paymentGatewayId": obj.get("paymentGatewayId"),
            "paymentMethodId": obj.get("paymentMethodId"),
            "paymentOption": [PaymentSchedulePaymentOptionFields.from_dict(_item) for _item in obj["paymentOption"]] if obj.get("paymentOption") is not None else None,
            "paymentScheduleNumber": obj.get("paymentScheduleNumber"),
            "period": obj.get("period"),
            "prepayment": obj.get("prepayment"),
            "runHour": obj.get("runHour"),
            "standalone": obj.get("standalone"),
            "startDate": obj.get("startDate"),
            "totalAmount": obj.get("totalAmount")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
