# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class CreateUsageRequest(BaseModel):
    """
    CreateUsageRequest
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    account_id: Optional[StrictStr] = Field(default=None, description=" The ID of the account associated with the usage data. This field is only required if no value is specified for the `AccountNumber` field. **Character limit**: 32 **Values**: a valid account ID.", alias="AccountId")
    account_number: Optional[StrictStr] = Field(default=None, description=" The number of the account associated with the usage data. This field is only required if no value is specified for the `AccountId` field. **Character limit**: 50 **Values**: a valid account number.", alias="AccountNumber")
    subscription_id: Optional[Annotated[str, Field(strict=True, max_length=32)]] = Field(default=None, description="The original ID of the subscription that contains the fees related to the usage data.   The ID of a subscription might change when you create amendments to the subscription. It is good practice to use the unique subscription number that you can specify in the `SubscriptionNumber` field. ", alias="SubscriptionId")
    subscription_number: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The unique identifier number of the subscription that contains the fees related to the usage data.  It is good practice to use this field when creating usage records. ", alias="SubscriptionNumber")
    charge_id: Optional[StrictStr] = Field(default=None, description=" The OrginalId of the rate plan charge related to the usage record, e.g., `2c9081a03c63c94c013c6873357a0117` **Character limit**: 32 **Values**: a valid rate plan charge OriginalID. ", alias="ChargeId")
    charge_number: Optional[Annotated[str, Field(strict=True, max_length=50)]] = Field(default=None, description="A unique number for the rate plan charge related to the usage record. For example, C-00000007. ", alias="ChargeNumber")
    start_date_time: datetime = Field(description=" The start date and time of a range of time when usage is tracked. Zuora uses this field value to determine the usage date. Unlike the `EndDateTime`, the `StartDateTime` field does affect usage calculation. **Character limit**: 29 **Values**: a valid date and time value", alias="StartDateTime")
    end_date_time: Optional[datetime] = Field(default=None, description=" The end date and time of a range of time when usage is tracked. Use this field for reporting; this field doesn't affect usage calculation. **Character limit**: 29 **Values**: a valid date and time value.", alias="EndDateTime")
    uom: StrictStr = Field(description=" Specifies the units to measure usage. Units of measure are configured in the web-based UI. Your values depend on your configuration in **Billing Settings**. **Character limit**: **Values**: a valid unit of measure", alias="UOM")
    quantity: Union[StrictFloat, StrictInt] = Field(description=" Indicates the number of units used. **Character limit**: 16 **Values**: a valid decimal amount equal to or greater than 0", alias="Quantity")
    description: Optional[Annotated[str, Field(strict=True, max_length=200)]] = Field(default=None, description="A description of the usage record. ", alias="Description")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["AccountId", "AccountNumber", "SubscriptionId", "SubscriptionNumber", "ChargeId", "ChargeNumber", "StartDateTime", "EndDateTime", "UOM", "Quantity", "Description"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of CreateUsageRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of CreateUsageRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "AccountId": obj.get("AccountId"),
            "AccountNumber": obj.get("AccountNumber"),
            "SubscriptionId": obj.get("SubscriptionId"),
            "SubscriptionNumber": obj.get("SubscriptionNumber"),
            "ChargeId": obj.get("ChargeId"),
            "ChargeNumber": obj.get("ChargeNumber"),
            "StartDateTime": obj.get("StartDateTime"),
            "EndDateTime": obj.get("EndDateTime"),
            "UOM": obj.get("UOM"),
            "Quantity": obj.get("Quantity"),
            "Description": obj.get("Description")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
