# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class GenerateBillingDocumentRequest(BaseModel):
    """
    GenerateBillingDocumentRequest
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    auto_post: Optional[StrictBool] = Field(default=False, description="Whether to automatically post the billing documents after the draft billing documents are generated.   If an error occurs during posting billing documents, the draft billing documents are not generated too. ", alias="autoPost")
    auto_renew: Optional[StrictBool] = Field(default=False, description="Whether to automatically renew the subscriptions with **Auto Renew** set to **Yes**.  ", alias="autoRenew")
    charge_type_to_exclude: Optional[List[StrictStr]] = Field(default=None, description="The types of the charges to be excluded from the generation of billing documents. The field values are case insensitive. Supported values include `onetime`, `recurring`, and `usage`.  ", alias="chargeTypeToExclude")
    credit_memo_reason_code: Optional[StrictStr] = Field(default=None, description="A code identifying the reason for the credit memo transaction that is generated by the request. The value must be an existing reason code. If you do not pass the field or pass the field with empty value, Zuora uses the default reason code.", alias="creditMemoReasonCode")
    effective_date: Optional[date] = Field(default=None, description="The date on which to generate the billing documents, in `yyyy-mm-dd` format. ", alias="effectiveDate")
    subscription_ids: Optional[List[StrictStr]] = Field(default=None, description="The IDs of the subscriptions that you want to create the billing documents for. Each value must be the ID of the latest version of an active subscription. ", alias="subscriptionIds")
    target_date: Optional[date] = Field(default=None, description="The date used to determine which charges are to be billed, in `yyyy-mm-dd` format. ", alias="targetDate")
    include_subscriptions: Optional[StrictBool] = Field(default=None, description="Whether to bill subscriptions. ", alias="includeSubscriptions")
    include_order_line_items: Optional[StrictBool] = Field(default=None, description="Whether to bill order line items. ", alias="includeOrderLineItems")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["autoPost", "autoRenew", "chargeTypeToExclude", "creditMemoReasonCode", "effectiveDate", "subscriptionIds", "targetDate", "includeSubscriptions", "includeOrderLineItems"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GenerateBillingDocumentRequest from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GenerateBillingDocumentRequest from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "autoPost": obj.get("autoPost") if obj.get("autoPost") is not None else False,
            "autoRenew": obj.get("autoRenew") if obj.get("autoRenew") is not None else False,
            "chargeTypeToExclude": obj.get("chargeTypeToExclude"),
            "creditMemoReasonCode": obj.get("creditMemoReasonCode"),
            "effectiveDate": obj.get("effectiveDate"),
            "subscriptionIds": obj.get("subscriptionIds"),
            "targetDate": obj.get("targetDate"),
            "includeSubscriptions": obj.get("includeSubscriptions"),
            "includeOrderLineItems": obj.get("includeOrderLineItems")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
