# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import date
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from zuora_sdk.models.fulfillment_state import FulfillmentState
from zuora_sdk.models.fulfillment_type import FulfillmentType
from zuora_sdk.models.get_fulfillment_item import GetFulfillmentItem
from typing import Optional, Set
from typing_extensions import Self

class GetFulfillment(BaseModel):
    """
    GetFulfillment
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    bill_target_date: Optional[date] = Field(default=None, description="The target date for the Fulfillment to be picked up by bill run for billing. ", alias="billTargetDate")
    carrier: Optional[StrictStr] = Field(default=None, description="The carrier of the Fulfillment. The available values can be configured in **Billing Settings** > **Fulfillment Settings** through Zuora UI. ")
    custom_fields: Optional[Dict[str, Any]] = Field(default=None, description="Container for custom fields of a Fulfillment object. ", alias="customFields")
    description: Optional[StrictStr] = Field(default=None, description="The description of the Fulfillment. ")
    exclude_item_billing_from_revenue_accounting: Optional[StrictBool] = Field(default=None, description="The flag to exclude Fulfillment related invoice items, invoice item adjustments, credit memo items, and debit memo items from revenue accounting.   **Note**: This field is only available if you have the [Zuora Billing - Revenue Integration](https://knowledgecenter.zuora.com/Zuora_Revenue/Zuora_Billing_-_Revenue_Integration) feature enabled.  ", alias="excludeItemBillingFromRevenueAccounting")
    exclude_item_booking_from_revenue_accounting: Optional[StrictBool] = Field(default=None, description="The flag to exclude Fulfillment from revenue accounting.  **Note**: This field is only available if you have the [Zuora Billing - Revenue Integration](https://knowledgecenter.zuora.com/Zuora_Revenue/Zuora_Billing_-_Revenue_Integration) feature enabled.  ", alias="excludeItemBookingFromRevenueAccounting")
    external_id: Optional[StrictStr] = Field(default=None, description="The external id of the Fulfillment. ", alias="externalId")
    fulfillment_date: Optional[date] = Field(default=None, description="The date of the Fulfillment. ", alias="fulfillmentDate")
    fulfillment_location: Optional[StrictStr] = Field(default=None, description="The fulfillment location of the Fulfillment. The available values can be configured in **Billing Settings** > **Fulfillment Settings** through Zuora UI. ", alias="fulfillmentLocation")
    fulfillment_system: Optional[StrictStr] = Field(default=None, description="The fulfillment system of the Fulfillment. The available values can be configured in **Billing Settings** > **Fulfillment Settings** through Zuora UI. ", alias="fulfillmentSystem")
    fulfillment_type: Optional[StrictStr] = Field(default=None, alias="fulfillmentType")
    order_line_item_id: Optional[StrictStr] = Field(default=None, description="The reference id of the related Order Line Item. ", alias="orderLineItemId")
    quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The quantity of the Fulfillment. ")
    state: Optional[StrictStr] = None
    tracking_number: Optional[StrictStr] = Field(default=None, description="The tracking number of the Fulfillment. ", alias="trackingNumber")
    fulfillment_items: Optional[List[GetFulfillmentItem]] = Field(default=None, description="The fulfillmentItems of the Fulfillment", alias="fulfillmentItems")
    fulfillment_number: Optional[StrictStr] = Field(default=None, description="The sytem generated number for the Fulfillment. ", alias="fulfillmentNumber")
    id: Optional[StrictStr] = Field(default=None, description="The system generated Id. ")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["billTargetDate", "carrier", "customFields", "description", "excludeItemBillingFromRevenueAccounting", "excludeItemBookingFromRevenueAccounting", "externalId", "fulfillmentDate", "fulfillmentLocation", "fulfillmentSystem", "fulfillmentType", "orderLineItemId", "quantity", "state", "trackingNumber", "fulfillmentItems", "fulfillmentNumber", "id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetFulfillment from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in fulfillment_items (list)
        _items = []
        if self.fulfillment_items:
            for _item_fulfillment_items in self.fulfillment_items:
                if _item_fulfillment_items:
                    _items.append(_item_fulfillment_items.to_dict())
            _dict['fulfillmentItems'] = _items
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetFulfillment from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "billTargetDate": obj.get("billTargetDate"),
            "carrier": obj.get("carrier"),
            "customFields": obj.get("customFields"),
            "description": obj.get("description"),
            "excludeItemBillingFromRevenueAccounting": obj.get("excludeItemBillingFromRevenueAccounting"),
            "excludeItemBookingFromRevenueAccounting": obj.get("excludeItemBookingFromRevenueAccounting"),
            "externalId": obj.get("externalId"),
            "fulfillmentDate": obj.get("fulfillmentDate"),
            "fulfillmentLocation": obj.get("fulfillmentLocation"),
            "fulfillmentSystem": obj.get("fulfillmentSystem"),
            "fulfillmentType": obj.get("fulfillmentType"),
            "orderLineItemId": obj.get("orderLineItemId"),
            "quantity": obj.get("quantity"),
            "state": obj.get("state"),
            "trackingNumber": obj.get("trackingNumber"),
            "fulfillmentItems": [GetFulfillmentItem.from_dict(_item) for _item in obj["fulfillmentItems"]] if obj.get("fulfillmentItems") is not None else None,
            "fulfillmentNumber": obj.get("fulfillmentNumber"),
            "id": obj.get("id")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
