# coding: utf-8

"""
    Zuora API Reference

    REST API reference for the Zuora Billing, Payments, and Central Platform! Check out the [REST API Overview](https://www.zuora.com/developer/api-references/api/overview/).

    The version of the OpenAPI document: 2024-05-20
    Contact: docs@zuora.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self

class UsageItem(BaseModel):
    """
    UsageItem
    """ # noqa: E501
    # avoid validation
    def __init__(self, **kwargs):
        # Directly assign values without validation using `construct()`
        model = self.construct(**kwargs)
        self.__dict__.update(model.__dict__)

        _dict_all = convert_snake_dict_to_camel(kwargs, self.model_fields, self.__properties)

        _dict = self.to_alias_dict(_dict_all)
        for field_name, field_info in self.model_fields.items():
            alias = field_info.alias or field_name
            if alias in _dict:
                self.__dict__[field_name] = _dict.get(alias)
        self.update_additional_properties(self, _dict_all)
        pass

    def __setattr__(self, name, value):
        # Override setattr to bypass validation when setting attributes
        object.__setattr__(self, name, value)

    id: Optional[StrictStr] = Field(default=None, description="Unique ID for the usage item. ")
    account_id: Optional[StrictStr] = Field(default=None, description="Customer account ID. ", alias="accountId")
    account_name: Optional[StrictStr] = Field(default=None, description="Customer account name. ", alias="accountName")
    account_number: Optional[StrictStr] = Field(default=None, description="Customer account number. ", alias="accountNumber")
    product_rate_plan_charge_number: Optional[StrictStr] = Field(default=None, description="Product rate plan charge number. **Note**: This field is only available if you have the [Dynamic Usage Charge] enabled. ", alias="productRatePlanChargeNumber")
    subscription_number: Optional[StrictStr] = Field(default=None, description="Number of the subscription covering this usage. ", alias="subscriptionNumber")
    charge_number: Optional[StrictStr] = Field(default=None, description="Number of the rate-plan charge that pays for this usage. ", alias="chargeNumber")
    start_date_time: Optional[StrictStr] = Field(default=None, description="Start date of the time period in which usage is tracked. Zuora uses this field value to determine the usage date. ", alias="startDateTime")
    unit_of_measure: Optional[StrictStr] = Field(default=None, description="Unit used to measure consumption. ", alias="unitOfMeasure")
    quantity: Optional[StrictStr] = Field(default=None, description="Number of units used. ")
    source_name: Optional[StrictStr] = Field(default=None, description="Source of the usage data. Possible values are: `Import`, `API`. ", alias="sourceName")
    file_name: Optional[StrictStr] = Field(default=None, description="The name of the import file when the usage record is imported from the file. ", alias="fileName")
    status: Optional[StrictStr] = Field(default=None, description="Possible values are: `Importing`, `Pending`, `Processed`. ")
    submission_date_time: Optional[StrictStr] = Field(default=None, description="Date when usage was submitted. ", alias="submissionDateTime")
    unique_key: Optional[StrictStr] = Field(default=None, description="a customer-defined specific identifier of a usage record.  **Note**: This field is only available if you have the [Prepaid with Drawdown](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown) or  [Unbilled Usage](https://knowledgecenter.zuora.com/Zuora_Billing/Bill_your_customers/Bill_for_usage_or_prepaid_products/Advanced_Consumption_Billing/Unbilled_Usage) feature enabled. See [Upload usage record with unique key](https://knowledgecenter.zuora.com/Billing/Billing_and_Payments/J_Billing_Operations/Prepaid_with_Drawdown/Prepaid_balance_transactions#Upload_usage_record_with_unique_key) for more information. ", alias="uniqueKey")
    additional_properties: Dict[str, Any] = {}
    __properties: ClassVar[List[str]] = ["id", "accountId", "accountName", "accountNumber", "productRatePlanChargeNumber", "subscriptionNumber", "chargeNumber", "startDateTime", "unitOfMeasure", "quantity", "sourceName", "fileName", "status", "submissionDateTime", "uniqueKey"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of UsageItem from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * Fields in `self.additional_properties` are added to the output dict.
        """
        excluded_fields: Set[str] = set([
            "additional_properties",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # puts key-value pairs in additional_properties in the top level
        if self.additional_properties is not None:
            for _key, _value in self.additional_properties.items():
                _dict[_key] = _value

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of UsageItem from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj = convert_snake_dict_to_camel(obj, cls.model_fields, cls.__properties)

        _obj = cls.model_validate(cls.to_alias_dict(obj))
        return cls.update_additional_properties(_obj, obj)

    @classmethod
    def update_additional_properties(cls, obj, _dict: Optional[Dict[str, Any]]):
        # store additional fields in additional_properties

        # store additional fields in additional_properties
        for _key in _dict.keys():
            if _key not in cls.__properties:
                obj.additional_properties[_key] = _dict.get(_key)


        return obj
        pass

    @classmethod
    def to_alias_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Dict[str, Any]]:

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return {
            "id": obj.get("id"),
            "accountId": obj.get("accountId"),
            "accountName": obj.get("accountName"),
            "accountNumber": obj.get("accountNumber"),
            "productRatePlanChargeNumber": obj.get("productRatePlanChargeNumber"),
            "subscriptionNumber": obj.get("subscriptionNumber"),
            "chargeNumber": obj.get("chargeNumber"),
            "startDateTime": obj.get("startDateTime"),
            "unitOfMeasure": obj.get("unitOfMeasure"),
            "quantity": obj.get("quantity"),
            "sourceName": obj.get("sourceName"),
            "fileName": obj.get("fileName"),
            "status": obj.get("status"),
            "submissionDateTime": obj.get("submissionDateTime"),
            "uniqueKey": obj.get("uniqueKey")
        }
        return _obj


def convert_snake_dict_to_camel(_dict: dict, model_fields, properties):
    if not isinstance(_dict, dict):
        return _dict
    new_dict = {}
    for k, v in _dict.items():
        if k in model_fields:
            # model_fields: key is attribute name like bill_to_contact,
            alias = model_fields.get(k).alias or k
            new_dict[alias] = v
            pass
        else:
            new_key = snake_to_camel(k)
            if properties is not None and isinstance(properties, list) and new_key in properties:
                new_dict[new_key] = v
            else:
                new_dict[k] = v
    return new_dict
    pass


def snake_to_camel(name):
    if name is None or '_' not in name:
        return name
    components = name.split('_')
    return components[0] + ''.join(x.title() for x in components[1:])
